'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});

var _typeof = typeof Symbol === "function" && typeof Symbol.iterator === "symbol" ? function (obj) { return typeof obj; } : function (obj) { return obj && typeof Symbol === "function" && obj.constructor === Symbol && obj !== Symbol.prototype ? "symbol" : typeof obj; };

exports.createSource = createSource;

var _immutable = require('immutable');

var _union = require('lodash/union');

var _union2 = _interopRequireDefault(_union);

var _difference = require('lodash/difference');

var _difference2 = _interopRequireDefault(_difference);

var _createSource2 = require('redux-source/lib/createSource');

var _createSource3 = _interopRequireDefault(_createSource2);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function _defineProperty(obj, key, value) { if (key in obj) { Object.defineProperty(obj, key, { value: value, enumerable: true, configurable: true, writable: true }); } else { obj[key] = value; } return obj; }

var combineMethods = {
  replace: function replace(_ref) {
    var state = _ref.state,
        stateName = _ref.stateName,
        result = _ref.result,
        entities = _ref.entities;

    state.mergeIn([stateName], {
      result: result,
      entities: entities
    });
  },
  merge: function merge(_ref2) {
    var state = _ref2.state,
        stateName = _ref2.stateName,
        prevResult = _ref2.prevResult,
        result = _ref2.result,
        entities = _ref2.entities;

    for (var resultName in result) {
      if (Array.isArray(result[resultName]) && (result[resultName].length === 0 || _typeof(result[resultName][0]) !== 'object')) {
        var merged = (0, _union2.default)(prevResult[resultName], result[resultName]);
        state.mergeIn([stateName, 'result'], _defineProperty({}, resultName, merged));
      } else {
        state.mergeIn([stateName, 'result'], _defineProperty({}, resultName, result[resultName]));
      }
    }
    for (var entityName in entities) {
      for (var entityId in entities[entityName]) {
        state.mergeIn([stateName, 'entities', entityName, entityId], entities[entityName][entityId]);
      }
    }
  },
  crop: function crop(_ref3) {
    var state = _ref3.state,
        stateName = _ref3.stateName,
        prevResult = _ref3.prevResult,
        result = _ref3.result;

    for (var resultName in result) {
      if (Array.isArray(result[resultName]) && (result[resultName].length === 0 || _typeof(result[resultName][0]) !== 'object')) {
        var filterd = (0, _difference2.default)(prevResult[resultName], result[resultName]);
        if (filterd.length !== prevResult[resultName].length) {
          state.mergeIn([stateName, 'result'], _defineProperty({}, resultName, filterd));
        }
      }
    }
  }
};

function createSource(_ref4) {
  var typeDefs = _ref4.schema,
      resolvers = _ref4.resolvers;

  return (0, _createSource3.default)({
    typeDefs: typeDefs,
    resolvers: resolvers,
    createInitialState: function createInitialState(_ref5) {
      var stateName = _ref5.stateName;
      return (0, _immutable.fromJS)(_defineProperty({}, stateName, {
        result: {},
        entities: {},
        isPending: false,
        pendingCount: 0,
        errors: []
      }));
    },
    pendingReducer: function pendingReducer(_ref6) {
      var stateName = _ref6.stateName;
      return function (state) {
        return state.withMutations(function (_state) {
          _state.mergeIn([stateName], {
            isPending: true,
            pendingCount: _state.getIn([stateName, 'pendingCount']) + 1
          });
          if (_state.getIn([stateName, 'errors']).size) {
            _state.setIn([stateName, 'errors'], (0, _immutable.fromJS)([]));
          }
        });
      };
    },
    successReducer: function successReducer(_ref7) {
      var stateName = _ref7.stateName,
          config = _ref7.config;
      return function (state, _ref8) {
        var _ref8$payload = _ref8.payload,
            payload = _ref8$payload === undefined ? {} : _ref8$payload;
        return state.withMutations(function (_state) {
          var prevResult = _state.getIn([stateName, 'result']);
          // const prevEntities = _state.getIn([stateName, 'entities']);
          var count = _state.getIn([stateName, 'pendingCount']) - 1;
          var combineMethod = combineMethods[config.combineResult || 'merge'];
          _state.mergeIn([stateName], {
            isPending: count > 0,
            pendingCount: count
          });
          combineMethod({
            state: _state,
            stateName: stateName,
            prevResult: prevResult.toJS(),
            // prevEntities: prevEntities.toJS(),
            result: payload.result || {},
            entities: payload.entities || {}
          });
          if (_state.getIn([stateName, 'errors']).size) {
            _state.setIn([stateName, 'errors'], (0, _immutable.fromJS)([]));
          }
        });
      };
    },
    errorReducer: function errorReducer(_ref9) {
      var stateName = _ref9.stateName;
      return function (state, _ref10) {
        var payload = _ref10.payload;

        var count = state.getIn([stateName, 'pendingCount']) - 1;
        return state.mergeIn([stateName], {
          isPending: count > 0,
          pendingCount: count,
          errors: payload
        });
      };
    }
  });
}
//# sourceMappingURL=index.js.map